"use strict";
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.rxToStream = void 0;
const stream = require("stream");
function rxToStream(src, options = { encoding: 'utf8' }, onError) {
    return new ReadableObservableStream(options, src, onError);
}
exports.rxToStream = rxToStream;
class ReadableObservableStream extends stream.Readable {
    constructor(options, _source, _onError) {
        super(streamOptions(options));
        this._source = _source;
        this._onError = _onError;
        this._isOpen = false;
        this._hasError = false;
        this._waiting = false;
        this._buffer = [];
        if (options.lazy === false) {
            this.subscribeIfNecessary();
        }
    }
    emitError() {
        this.emit('error', this._error);
        if (this._onError) {
            this._onError(this._error, this);
        }
    }
    _destroy() {
        if (this._subscription) {
            this._subscription.unsubscribe();
        }
    }
    _read() {
        const { _buffer } = this;
        this.subscribeIfNecessary();
        if (_buffer.length > 0) {
            while (_buffer.length > 0) {
                this._waiting = this.push(_buffer.shift());
                if (!this._waiting)
                    break;
            }
        }
        else {
            if (this._isOpen) {
                this._waiting = true;
            }
            else {
                if (this._hasError) {
                    this.emitError();
                }
                else {
                    this.push(null);
                }
            }
        }
    }
    subscribeIfNecessary() {
        if (this._subscription)
            return;
        const { _buffer } = this;
        this._isOpen = true;
        this._waiting = true;
        this._subscription = this._source.subscribe({
            next: (value) => {
                if (this._waiting) {
                    this._waiting = this.push(value);
                }
                else {
                    _buffer.push(value);
                }
            },
            error: (err) => {
                this._isOpen = false;
                this._hasError = true;
                this._error = err;
                if (this._waiting) {
                    this.emitError();
                }
            },
            complete: () => {
                this._isOpen = false;
                if (this._waiting) {
                    this.push(null);
                }
            },
        });
    }
}
function streamOptions(options) {
    const { lazy: _ } = options, streamOptions = __rest(options, ["lazy"]);
    return streamOptions;
}
//# sourceMappingURL=rxToStream.js.map